/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.pdfconv;

import jcmdline.*;

import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import java.util.Properties;

public class PDFConv
{
    private static final String VERSION="0.2.2";

    Converter _conv=null;
    LogHandler _logger=null;

    public static void main(String[] args)
    {
        try
        {
            PDFConv pc=new PDFConv();
            pc.readParameters(args);
            pc.convert();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.exit(1);
        }
        System.exit(0);
    }

    public PDFConv()
    {
        _conv=new Converter();
    }

    private void readParameters(String[] args)
    {
        String helpText="This program reads a PDF file, and saves each page as an image (JPEG or PICT). Additionaly, a HTML, "+
            "BookMaker source file or NTK book source file is written to reference all files.";

        FileParam sourceParam=new FileParam("source", "the PDF input file(s)", FileParam.IS_FILE&FileParam.IS_READABLE, FileParam.REQUIRED, FileParam.MULTI_VALUED);
        StringParam baseNameParam=new StringParam("basename", "the base name for all output files", true);
        StringParam pdfScaleParam=new StringParam("pdfscale", "the scale factor for generating the internal images from PDF (float value)", true);
        IntParam widthParam=new IntParam("maxwidth", "maximum width of a resulting image", 50, 1000, true);
        IntParam heigthParam=new IntParam("maxheigth", "maximum heigth of a resulting image", 50, 1000, true);
        StringParam scaleFactorParam=new StringParam("scalefactor", "how to scale the created images (float value)", true);
        FileParam targetDirParam=new FileParam("targetdir", "where to write the resulting files", FileParam.IS_DIR&FileParam.IS_WRITEABLE, FileParam.OPTIONAL, FileParam.SINGLE_VALUED);
        IntParam qualityParam=new IntParam("quality", "quality of a resulting JPEG image", 10, 100, true);
        BooleanParam pictParam=new BooleanParam("writePICT", "write PICT image files");
        BooleanParam bmpParam=new BooleanParam("writeBMP", "write BMP image files (gray)");
        BooleanParam bmp2Param=new BooleanParam("writeBMP2", "write BMP image files (black&white)");
        BooleanParam jpegParam=new BooleanParam("writeJPEG", "write JPG image files");
        BooleanParam bookParam=new BooleanParam("writeBook", "write BookMaker file");
        BooleanParam ntkParam=new BooleanParam("writeNTK", "write NTK Book file");
        BooleanParam htmlParam=new BooleanParam("writeHTML", "write HTML file");
        BooleanParam verboseParam=new BooleanParam("verbose", "be verbose");
        BooleanParam debugParam=new BooleanParam("debug", "be very verbose - give debugging information");
        StringParam pagesParam=new StringParam("pages","which pages to convert (page:page:from-to:from-:-to)",true);

        String targetBaseName=null;
        String targetDir="";

        CmdLineHandler cl=
            new VersionCmdLineHandler("V "+VERSION,
                new HelpCmdLineHandler(helpText,
                    "run",
                    "convert PDF files for showing on a Newton",
                    new Parameter[]{baseNameParam, pictParam, bmpParam, bmp2Param, jpegParam, bookParam, ntkParam, htmlParam, widthParam,
                                    heigthParam, scaleFactorParam, targetDirParam, qualityParam, pagesParam, verboseParam, debugParam},
                    new Parameter[]{sourceParam},
                    new PosixCmdLineParser()));
        cl.parse(args);
        
        File f=new File ("pdfconv.cfg");
        if (f.exists())
        {
            Properties p=new Properties();
            try
            {
                p.load(new FileInputStream(f));
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            try
            {
                String s=p.getProperty("basename");
                if (null!=s&&0!=s.length()&&!baseNameParam.isSet())
                    baseNameParam.setValue(s);
                s=p.getProperty("maxwidth");
                if (null!=s&&0!=s.length()&&!widthParam.isSet())
                    widthParam.setValue(s);
                s=p.getProperty("maxheigth");
                if (null!=s&&0!=s.length()&&!heigthParam.isSet())
                    heigthParam.setValue(s);
                s=p.getProperty("scalefactor");
                if (null!=s&&0!=s.length()&&!scaleFactorParam.isSet())
                    scaleFactorParam.setValue(s);
                s=p.getProperty("targetdir");
                if (null!=s&&0!=s.length()&&!targetDirParam.isSet())
                    targetDirParam.setValue(s);
                s=p.getProperty("quality");
                if (null!=s&&0!=s.length()&&!qualityParam.isSet())
                    qualityParam.setValue(s);
                s=p.getProperty("verbose");
                if (null!=s&&0!=s.length()&&!verboseParam.isSet())
                    verboseParam.setValue("true");
                s=p.getProperty("debug");
                if (null!=s&&0!=s.length()&&!debugParam.isSet())
                    debugParam.setValue("true");
                s=p.getProperty("texttarget");
                if (null!=s&&0!=s.length())
                {
                    while (true)
                    {
                        int pos=s.indexOf(',');
                        String target=s;
                        if (-1!=pos)
                        {
                            target=s.substring(0,pos);
                            s=s.substring(pos+1);
                        }
                        target=target.toLowerCase();
                        if (target.equals("html")&&!htmlParam.isSet())
                            htmlParam.setValue("true");
                        else if (target.equals("book")&&!bookParam.isSet())
                            bookParam.setValue("true");
                        else if (target.equals("ntk")&&!ntkParam.isSet())
                            ntkParam.setValue("true");
                        if (-1==pos)
                            break;
                    }
                }
                s=p.getProperty("imagetarget");
                if (null!=s&&0!=s.length())
                {
                    while (true)
                    {
                        int pos=s.indexOf(',');
                        String target=s;
                        if (-1!=pos)
                        {
                            target=s.substring(0,pos);
                            s=s.substring(pos+1);
                        }
                        target=target.toLowerCase();
                        if (target.equals("bmp")&&!bmpParam.isSet())
                            bmpParam.setValue("true");
                        else if (target.equals("bmp2")&&!bmp2Param.isSet())
                            bmp2Param.setValue("true");
                        else if (target.equals("jpeg")&&!jpegParam.isSet())
                            jpegParam.setValue("true");
                        else if (target.equals("pict")&&!pictParam.isSet())
                            pictParam.setValue("true");
                        if (-1==pos)
                            break;
                    }
                }
                s=p.getProperty("pages");
                if (null!=s&&0!=s.length()&&!pagesParam.isSet())
                    pagesParam.setValue(s);
                s=p.getProperty("pdfscale");
                if (null!=s&&0!=s.length()&&!pdfScaleParam.isSet())
                    pdfScaleParam.setValue(s);
            }
            catch (CmdLineException e)
            {
                System.err.println("error in configfile: "+e.getMessage());
                System.exit(1);
            }
        }
        
        targetDir=targetDirParam.getValue();
        targetBaseName=baseNameParam.getValue();

        if (null!=targetDir&&0!=targetDir.length())
        {
            targetDir=targetDir.replace('\\', '/');
            if (!targetDir.endsWith("/"))
                targetDir+="/";
            targetDir=targetDir.trim();
            if (0!=targetDir.length())
            {
                _conv.setTargetDir(targetDir);
            }
        }

        if (null!=targetBaseName)
        {
            targetBaseName=targetBaseName.trim();
            if (0!=targetBaseName.length())
                _conv.setBaseName(targetBaseName);
        }

        if (pdfScaleParam.isSet())
        {
            try
            {
                _conv.setPDFScale(Float.parseFloat(pdfScaleParam.getValue()));
            }
            catch (NumberFormatException e)
            {
                cl.exitUsageError("'-pdfscale' must be a float value");
            }
        }

        readImagesParameter(pictParam, bmpParam, bmp2Param, cl, jpegParam, htmlParam, widthParam, heigthParam, scaleFactorParam, qualityParam);

        if (bookParam.isTrue())
        {
            if (pictParam.isTrue())
                _conv.setWriteBookMac();
            if (bmpParam.isTrue()||bmp2Param.isTrue())
                _conv.setWriteBookWin();
            if (!pictParam.isTrue()&&!bmpParam.isTrue()&&!bmp2Param.isTrue())
            {
                _conv.setWriteBookMac();
                _conv.setWriteBookWin();
            }
        }

        if (ntkParam.isTrue())
        {
            if (pictParam.isTrue())
                _conv.setWriteNTKMac();
            if (bmpParam.isTrue()||bmp2Param.isTrue())
                _conv.setWriteNTKWin();
            if (!pictParam.isTrue()&&!bmpParam.isTrue()&&!bmp2Param.isTrue())
            {
                _conv.setWriteNTKMac();
                _conv.setWriteNTKWin();
            }
        }

        Collection files=sourceParam.getFiles();
        for (Iterator iterator=files.iterator(); iterator.hasNext();)
        {
            File file=(File)iterator.next();
            _conv.addSourceFile(file);
        }

        _logger=new DefaultLogger(verboseParam.isTrue()||debugParam.isTrue(),debugParam.isTrue());
        
        String pages=pagesParam.getValue();
        if (null!=pages&&0!=pages.length())
        {
            readPagesParameter(pages, cl);
        }
    }

    private void readImagesParameter(BooleanParam pictParam, BooleanParam bmpParam, BooleanParam bmp2Param, CmdLineHandler cl, BooleanParam jpegParam, BooleanParam htmlParam, IntParam widthParam, IntParam heigthParam, StringParam scaleFactorParam, IntParam qualityParam)
    {
        if (pictParam.isTrue())
            _conv.setWritePICT();
        if (bmpParam.isTrue()&&bmp2Param.isTrue())
        {
            cl.exitUsageError("'-"+bmpParam.getTag()+"' and '-"+bmp2Param.getTag()+"' cannot used both!");
        }
        if (bmpParam.isTrue())
            _conv.setWriteBMPGray();
        if (bmp2Param.isTrue())
            _conv.setWriteBMPBW();
        if (jpegParam.isTrue())
            _conv.setWriteJPEG();
        if (htmlParam.isTrue())
            _conv.setWriteHTML();

        if (widthParam.isSet())
        {
            _conv.setMaxWidth(widthParam.intValue());
        }
        if (heigthParam.isSet())
            _conv.setMaxHeigth(heigthParam.intValue());
        if (scaleFactorParam.isSet())
        {
            try
            {
                _conv.setScaleFactor(Double.parseDouble(scaleFactorParam.getValue()));
            }
            catch (NumberFormatException e)
            {
                cl.exitUsageError("'-scalefactor' must be a Double value");
            }
        }
        if (qualityParam.isSet())
        {
            if (!jpegParam.isTrue())
                cl.exitUsageError("'-quality' can only be used when writing JPEG files.");
            _conv.setQuality(qualityParam.intValue());
        }
    }

    private void readPagesParameter(String pages, CmdLineHandler cl)
    {
        Vector v=new Vector();
        while (true)
        {
            int pos=pages.indexOf(':');
            String currentPage=pages;
            if (-1!=pos)
            {
                currentPage=pages.substring(0,pos);
                pages=pages.substring(pos+1);
            }
            try
            {
                int pos2=currentPage.indexOf('-');
                if (-1==pos2)
                {
                    int page=Integer.parseInt(currentPage);
                    if (0==page)
                        cl.exitUsageError("invalid page: 0 is not allowed!");
                    v.addElement(new Integer(page));
                }
                else if (0==pos2)
                {
                    currentPage=currentPage.substring(1);
                    int page=Integer.parseInt(currentPage);
                    if (0==page)
                        cl.exitUsageError("invalid page: 0 is not allowed!");
                    for (int i=1;i<=page;i++)
                        v.addElement(new Integer(i));
                }
                else if ((currentPage.length()-1)==pos2)
                {
                    v.addElement(currentPage);
                }
                else
                {
                    String firstPageStr=currentPage.substring(0,pos2);
                    String lastPageStr=currentPage.substring(pos2+1);
                    int firstPage=Integer.parseInt(firstPageStr);
                    int lastPage=Integer.parseInt(lastPageStr);
                    if (lastPage<=firstPage)
                        cl.exitUsageError("pages: range must be in correct order!");
                    for (int i=firstPage;i<=lastPage;i++)
                        v.addElement(new Integer(i));
                }
                if (-1==pos)
                    break;
            }
            catch (NumberFormatException e)
            {
                cl.exitUsageError("pages: invalid number given ("+currentPage+")");
            }
        }

        _conv.setPagesToConvert(v);
    }

    public void convert() throws IOException
    {
        _conv.convert(_logger);
    }
}
