package de.hendriklipka.pdfconv.gui;

import de.hendriklipka.pdfconv.Converter;

import java.io.File;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.util.Vector;

public class GUILogicHandler implements LogicHandler
{
    private WizardComponent _wizard=null;

    private ImageOptionsPage _imageOptionsPage=null;
    private OutputOptionsPage _outputOptionsPage=null;
    private FileTypesPage _fileTypesPage=null;
    private SelectFilesPage _selectFilesPage=null;
    private SelectActionPage _selectActionPage=null;
    
    private ShowProgressPage _progressPage=null;
    
    private boolean _showCustomActions=false;
    private Map _data=new HashMap();

    public GUILogicHandler(WizardComponent wizardComponent, ImageOptionsPage imageOptionsPage, OutputOptionsPage outputOptionsPage,FileTypesPage fileTypesPage,
                           SelectFilesPage selectFilesPage, SelectActionPage selectActionsPage, ShowProgressPage showProgressPage)
    {
        _wizard=wizardComponent;
        _imageOptionsPage=imageOptionsPage;
        _outputOptionsPage=outputOptionsPage;
        _fileTypesPage=fileTypesPage;
        _selectFilesPage=selectFilesPage;
        _selectActionPage=selectActionsPage;
        _progressPage=showProgressPage;
    }

    public void setConfigurationData(String name, String value)
    {
        if (null==value||0==value.length())
            removeConfigurationData(name);
        else
            _data.put(name, value);
    }

    public void removeConfigurationData(String name)
    {
        _data.remove(name);
    }

    private boolean isDataSet(String name)
    {
        String s = (String) _data.get(name);
        return (null != s && 0 != s.length());
    }

    private String getData(String name)
    {
        return (String) _data.get(name);
    }

    private int getIntData(String name, int def)
    {
        String s = (String) _data.get(name);
        if (null == s || 0 == s.length())
            return def;
        try
        {
            return Integer.parseInt(s);
        }
        catch (NumberFormatException e)
        {
            return def;
        }
    }

    private boolean getBooleanData(String name, boolean def)
    {
        String s = (String) _data.get(name);
        if (null==s||0==s.length())
            return def;
        return (s.equalsIgnoreCase("true"));
    }

    public void enableAdvancedOptions(boolean enable)
    {
        _imageOptionsPage.setAdvancedOptions(enable);
        _outputOptionsPage.setAdvancedOptions(enable);
    }

    public void enableCustomActions(boolean enable)
    {
        if (enable)
        {
            _fileTypesPage.setEnabled(true);
            _showCustomActions=true;
        }
        else
        {
            _fileTypesPage.setEnabled(false);
            _showCustomActions=false;
        }
    }

    public void setImageGenerationType(String type)
    {
        _imageOptionsPage.setDesiredImageType(type);
    }

    public void setFileGenerationType(String type)
    {
        _fileTypesPage.setDesiredFileType(type);
    }

    public void changeFinishState()
    {
        boolean allowed=true;
        if (!_outputOptionsPage.allowFinish())
            allowed=false;
        if (!_selectFilesPage.allowFinish())
            allowed=false;
        if (_showCustomActions)
        {
            if (!_fileTypesPage.allowFinish())
                allowed=false;
            if (!_imageOptionsPage.allowFinish())
                allowed=false;
        }
        _wizard.changeFinishState(allowed);
    }

    public void finishClicked()
    {
        _fileTypesPage.setEnabled(false);
        _imageOptionsPage.setEnabled(false);
        _outputOptionsPage.setEnabled(false);
        _selectFilesPage.setEnabled(false);
        _selectActionPage.setEnabled(false);

        // make sure all pages write their data
        _fileTypesPage.leavePage();
        _imageOptionsPage.leavePage();
        _outputOptionsPage.leavePage();
        _selectFilesPage.leavePage();
        _selectActionPage.leavePage();

        _progressPage.setEnabled(true);
        _progressPage.showMessages(getBooleanData(PageConstants.SHOW_MESSAGES,false));
        _progressPage.setVerbose(getBooleanData(PageConstants.SHOW_VERBOSE, false));
        _progressPage.setDebug(getBooleanData(PageConstants.SHOW_DEBUG, false));

        _wizard.gotoPage(_progressPage);
        _wizard.goIntoFinalState();
        try
        {
            SwingWorker worker=new SwingWorker()
            {
                public Object construct()
                {
                    try
                    {
                        generateFiles();
                    }
                    catch (Exception e)
                    {
                        System.err.println("catched error: "+e.toString());
                    }
                    return null;
                }
            }; 
            worker.start();
        }
        catch (Exception e)
        {
            _progressPage.printError(e.toString());
            e.printStackTrace();
        }
        _wizard.goIntoFinishedState();
    }

    private void generateFiles()
    {
        try
        {
            Converter conv=new Converter();
            conv.setTargetDir(_outputOptionsPage.getTargetDirectory());
            conv.setBaseName(_outputOptionsPage.getBaseName());
            Enumeration files=_selectFilesPage.getFiles();
            while (files.hasMoreElements())
            {
                String file=(String)files.nextElement();
                conv.addSourceFile(new File(file));

                conv.setBaseName(getData(PageConstants.FILE_BASENAME));
                conv.setTargetDir(getData(PageConstants.FILE_OUTDIR));

                if (isDataSet(PageConstants.IMG_SCALE_MAX_HEIGTH)) conv.setMaxHeigth(getIntData(PageConstants.IMG_SCALE_MAX_HEIGTH,-1));
                if (isDataSet(PageConstants.IMG_SCALE_MAX_WIDTH)) conv.setMaxWidth(getIntData(PageConstants.IMG_SCALE_MAX_WIDTH,-1));
                if (isDataSet(PageConstants.IMG_SCALE_FACTOR)) conv.setScaleFactor(getIntData(PageConstants.IMG_SCALE_FACTOR,-1));
                if (isDataSet(PageConstants.IMG_SCALE_INTERNAL)) conv.setPDFScale(getIntData(PageConstants.IMG_SCALE_INTERNAL,-1));

                if (getBooleanData(PageConstants.IMG_TYPE_BMP,false)) conv.setWriteBMPGray();
                if (getBooleanData(PageConstants.IMG_TYPE_BMP2, false)) conv.setWriteBMPBW();
                if (getBooleanData(PageConstants.IMG_TYPE_JPEG, false)) conv.setWriteJPEG();
                if (getBooleanData(PageConstants.IMG_TYPE_PICT, false)) conv.setWritePICT();

                if (getBooleanData(PageConstants.FILE_TYPE_BOOK_MAC, false)) conv.setWriteBookMac();
                if (getBooleanData(PageConstants.FILE_TYPE_BOOK_WIN, false)) conv.setWriteBookWin();
                if (getBooleanData(PageConstants.FILE_TYPE_NTK_MAC, false)) conv.setWriteNTKMac();
                if (getBooleanData(PageConstants.FILE_TYPE_NTK_WIN, false)) conv.setWriteNTKWin();
                if (getBooleanData(PageConstants.FILE_TYPE_HTML, false)) conv.setWriteHTML();

                if (isDataSet(PageConstants.PAGES)) conv.setPagesToConvert(readPagesParameter(getData(PageConstants.PAGES)));
            }
            conv.convert(_progressPage);
        }
        catch (Throwable e)
        {
            e.printStackTrace();
            _progressPage.printError(e.toString());
        }
    }

    public void cancelClicked()
    {
        System.exit(0);
    }

    private Vector readPagesParameter(String pages)
    {
        Vector v = new Vector();
        while (true)
        {
            int pos = pages.indexOf(':');
            String currentPage = pages;
            if (-1 != pos)
            {
                currentPage = pages.substring(0, pos);
                pages = pages.substring(pos + 1);
            }
            try
            {
                int pos2 = currentPage.indexOf('-');
                if (-1 == pos2)
                {
                    int page = Integer.parseInt(currentPage);
                    if (0 == page)
                    {
                        //@todo error handling
                    }
                    v.addElement(new Integer(page));
                }
                else if (0 == pos2)
                {
                    currentPage = currentPage.substring(1);
                    int page = Integer.parseInt(currentPage);
                    if (0 == page)
                    {
                        //@todo error handling
                    }
                    for (int i = 1; i <= page; i++)
                        v.addElement(new Integer(i));
                }
                else if ((currentPage.length() - 1) == pos2)
                {
                    v.addElement(currentPage);
                }
                else
                {
                    String firstPageStr = currentPage.substring(0, pos2);
                    String lastPageStr = currentPage.substring(pos2 + 1);
                    int firstPage = Integer.parseInt(firstPageStr);
                    int lastPage = Integer.parseInt(lastPageStr);
                    if (lastPage <= firstPage)
                    {
                        //@todo error handling
                    }
                    for (int i = firstPage; i <= lastPage; i++)
                        v.addElement(new Integer(i));
                }
                if (-1 == pos)
                    break;
            }
            catch (NumberFormatException e)
            {
                {
                    //@todo error handling
                }
            }
        }
        return v;
    }
}
